/*
	For reading and parsing the OPML
	file into PancakeJava and provide
	general XML functions
	
	Essential reading:
	http://www.java2s.com/ExampleCode/J2ME/FileConnection.htm
	http://www.microjava.com/articles
	http://www-128.ibm.com/developerworks/library/wi-parsexml/
	
	Author: filipp.lepalaan@evtek.fi
*/

import java.util.Vector;
import java.io.*;
import javax.microedition.io.*;
import javax.microedition.rms.*;
import org.kxml.*;
import org.kxml.kdom.*;
import org.kxml.parser.*;

class Feeder extends RSSParser {

	private RecordStore rs;
	private InputStream is;
	private boolean debug;

public Feeder (InputStream feeds) throws IOException
{
	super ();
	debug = false;
	rs = null;
	is = feeds;
	
	try  {
		rs = RecordStore.openRecordStore ("feeds", true);
		
		if (debug) System.err.println ("Stores: " + rs.listRecordStores ());
	
	} catch (RecordStoreException rse) {
		System.err.println ("Exception: " + rse);
		rse.printStackTrace ();
	}
}

/*
	Import OPML file
	We're using a pull parser, so have to loop through the whole thing.
	It's also very particular about the order of elements, so be careful!
	TODO!
	- (DONE) Skip tags inside the description
	- Completely ignoring Atom feeds (for the time being) would be nice.
*/

public void loadFeeds () throws IOException
{
	XmlParser parser = new XmlParser (new InputStreamReader (is));

	if (debug) {
		byte b[] = new byte[1];
		StringBuffer str = new StringBuffer ();
		while (is.read (b) != -1) { str.append (new String (b)); }
		System.err.println (str);
	}

	String title, link, description;
	title = link = description = null;
	
	Document doc = new Document ();
	doc.parse (parser);
	Element root = doc.getRootElement ();
	Element feeds = root.getElement ("body");
	int count = feeds.getChildCount ();
	
	for (int i = 0; i < count; i++) {
		if (feeds.getType (i) == Xml.ELEMENT) {
			Element el = feeds.getElement (i);
			title = el.getValue ("title");
			link = el.getValue ("xmlUrl");
			description = el.getValue ("description");
			mRSSListener.itemParsed (title, link, description);
			if (debug) {
				System.err.println ("Element name: " + el.getName ());
				System.err.println ("\t" + title);
				System.err.println ("\t" + link);
				System.err.println ("\t" + description );
			}
		}
	}
	
	is.close ();
	
}

/*
	(DEPRECATED) Return url of a feed. Each should be unique, so return the first we find.
	Don't really need this anymore because I know how to work with files! :P
*/
public String getFeed (String name)
{
	String result = null;
	
	try {
	// No filters or comparators, this is also faster.
	RecordEnumeration re = rs.enumerateRecords (null, null, true);
	
	while (re.hasNextElement ())
	{
		int id = re.nextRecordId ();
		ByteArrayInputStream bais = new ByteArrayInputStream (rs.getRecord (id));
		DataInputStream dis = new DataInputStream (bais);
		try {
			if (name.equals (dis.readUTF ())) { result = dis.readUTF (); }
		} catch (EOFException eofx) {
			System.err.println (eofx);
			eofx.printStackTrace ();
			}
	}
	
	} catch (Exception e) {
		System.err.println (e);
		e.printStackTrace ();
		}
		
	return result;
}

// (DEPRECATED) Add newsfeed to persistant storage
public void addFeed (String name, String url)
{
	int recId;
	ByteArrayOutputStream baos = new ByteArrayOutputStream ();
	DataOutputStream outStream = new DataOutputStream (baos);
	try {
		outStream.writeUTF (name);
		outStream.writeUTF (url);
	} catch (IOException iox) {
		System.err.println (iox);
		iox.printStackTrace ();
	}
	
	byte[] b = baos.toByteArray ();
	try {
		recId = rs.addRecord (b, 0, b.length);
	} catch (RecordStoreException rsx) {
		System.err.println (rsx);
		rsx.printStackTrace ();
		}
	}
}