/*
	Parts of this from sun.com
	@version 0.7
	@author filipp.lepalaan@evtek.fi
	@date 02.03.2006
	
	Extra reading
	- http://today.java.net/pub/a/today/2005/09/27/j2me4.html?page=2
	- http://developers.sun.com/techtopics/mobility/midp/articles/threading/
	- file://localhost/Users/filipp/Documents/Code/midp2/tutorial/index.html
	- http://mpowerplayer.com/products-sdk.php
	- http://developers.sun.com/techtopics/mobility/midp/questions/res/
	- http://www.wirelessdevnet.com/columns/feb2001/editor14.html
	- file:///Users/filipp/Documents/Code/midp2/javadoc/index.html
	
	TODO:
	- Use a Form, itemStateListener, ChoiceGroup (+OPML groups?)
	- (DEPRECATED) Record Management System (RMS) for persistant store
	- (BONUS) Localisation & Resources, strip tags
	- Tag stripping is a joke. :)
	- FIX crashes
	- Decode HTML entities
*/

import java.io.*;
import java.util.Vector;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

public class PancakeJava extends MIDlet implements CommandListener, RSSListener
{
	private Feeder feeder;
	private Display disp;
	private List feedsList, titlesList;						// The graphic lists
	private Command exitComm, detailsComm;					// Only two buttons
	private Ticker ticker;
	private InputStream feeds;								// OPML file
	private boolean initialised, browsingTitles;			// Keeping track of state
	private Vector titles, descriptions, links;				// The data for the lists
	private Screen waitScreen;
  
public PancakeJava ()
{
	initialised = false;
  	browsingTitles = false;									// Always start with feeds
  	
    titles = new Vector ();									// one = two = new Vector () gives weird results!
    links = new Vector ();
    descriptions = new Vector ();
    
  	try {
		feeds = this.getClass ().getResourceAsStream ("/MySubscriptions.opml");
		feeder = new Feeder (feeds);
	} catch (IOException iox) {
		System.err.println ("Couldn't open OPML file. \n" + iox);
		iox.printStackTrace ();
		}
		
	initComponents ();
  
}

/*
	Do basic setup, show waitscreen, load objects etc.
*/
public void startApp ()
{
    if (disp == null) disp = Display.getDisplay (this);
    
    if (! initialised)
	{
      waitScreen = new Form ("Connecting...");
      disp.setCurrent (waitScreen);
      feeder.setRSSListener (this);
      
      try  { feeder.loadFeeds ();	}
      catch (IOException iox) { System.err.println (iox); iox.printStackTrace (); }

      initialised = true;
    }
    
    else disp.setCurrent (titlesList);
  
  }
  
/*
	Make this whole thing respond to commands.
	We have two modes of operation  - feed list and titles per feed
	exitComm can mean both quitting the application or going back to the previous list.
	TODO! int selection is off!
*/
public void commandAction(Command c, Displayable s)
{ 
    if (c == exitComm) {
    	if (browsingTitles) 
    		{ disp.setCurrent (feedsList); browsingTitles = false; }
    	else
    		notifyDestroyed ();
    }
    
    else if (c == detailsComm || c == List.SELECT_COMMAND)
	{
      int selection = (browsingTitles) ? titlesList.getSelectedIndex () : feedsList.getSelectedIndex ();
      if (selection == -1) return;
      
      String url = (String) links.elementAt (selection);
      String title = (String) titles.elementAt (selection);
      String description = (String) descriptions.elementAt (selection);
      
      String[] rssElements = { "title", "link", "description" };
      ticker.setString (description);
      
      if (browsingTitles) {
		  Alert a = new Alert (title, description, null, null);
		  a.setTimeout (Alert.FOREVER);
		  disp.setCurrent (a, titlesList);
      } else {
      		descriptions = new Vector ();				// Changing modes, reset titles and descriptions
      		titles = new Vector ();
      		titlesList.deleteAll ();
			disp.setCurrent (titlesList);
			feeder.parse (url, rssElements);
			browsingTitles = true;
      	}
    }
    else
		{ System.err.println (c.getLabel ()); }
  }

/*
	The connection between our UI and backend code.
	Called every time an important element is parsed
	in either OPML or RSS
*/
public void itemParsed (String title, String link, String description)
{
    titles.addElement (title);
    descriptions.addElement (description);
    links.addElement (link);
    
    if (browsingTitles) {
    	titlesList.append (title, null);
    	disp.setCurrent (titlesList);
    } else {
    	feedsList.append (title, null);
		disp.setCurrent (feedsList);
    	}
}

/*
	Setup UI elements and commandlisteners
*/
private void initComponents ()
{
	// Create the title list.
	ticker = new Ticker ("Welcome to PancakeJava");
	titlesList = new List ("Pick a headline", List.IMPLICIT);
	titlesList.setTicker (ticker);
	
	feedsList = new List ("Pick a feed", List.IMPLICIT);
	feedsList.setTicker (ticker);
	
	exitComm = new Command ("Exit", Command.EXIT, 0);
	detailsComm = new Command ("Next", Command.SCREEN, 0);
	
	titlesList.addCommand (exitComm);
	titlesList.addCommand (detailsComm);
	feedsList.addCommand (detailsComm);
	feedsList.addCommand (exitComm);
	
	titlesList.setCommandListener (this);
	feedsList.setCommandListener (this);
}

public void exception (java.io.IOException ioe)
{
    Alert a = new Alert ("Exception", ioe.toString(), null, null);
    a.setTimeout(Alert.FOREVER);
    disp.setCurrent(a, titlesList);
}

public void pauseApp () {}
public void destroyApp (boolean unconditional) {}
}