/*
	Some of this from
	http://developers.sun.com/techtopics/mobility/midp/articles/parsingxml/
	
	XML Parsing provided by
	http://kxml.objectweb.org/software/downloads/index.html
	http://kxml.objectweb.org/software/documentation/apidocs/
*/

import java.io.*;
import javax.microedition.io.*;

import org.kxml.*;
import org.kxml.parser.*;
import org.kxml.kdom.*;

public class RSSParser
{
  protected RSSListener mRSSListener;
  private boolean debug;
  
  // Non-blocking worker that calls a thread to do the dirty work.
  public void parse (final String url, final String[] args)
  {
  	debug = false;
    Thread t = new Thread () {
      public void run () {
        // set up the network connection
        HttpConnection hc = null;
        try {
          hc = (HttpConnection)Connector.open (url);
		  hc.setRequestProperty ("User-Agent", "Profile/MIDP-2.0 Configuration/CLDC-1.0");
          parse (hc.openInputStream (), args);
        }
        catch (IOException ioe) {
          mRSSListener.exception (ioe);
        }
        finally {
          try { if (hc != null) hc.close (); }
          catch (IOException ignored) {}
        }
      }
    };
    
    t.start();
    
  }

// Blocking. Would be nice to generalise this and have it parse
// other XML files. First, the number of arguments should be made arbitrary
public void parse (InputStream in, String[] args) throws IOException
{
  	Reader reader = new InputStreamReader (in);
  	XmlParser parser = new XmlParser (reader);
  	String title, link, description;
  	title = link = description = null;
  	Document doc = new Document ();
  	doc.parse (parser);
	Element root = doc.getRootElement ();
	Element items = root.getElement ("channel");
	int count = items.getChildCount ();
	
	for (int i = 0; i < count; i++) {
		if (items.getType (i) == Xml.ELEMENT) {
			Element el = items.getElement (i);
			if (el.getName().equals ("item")) {
				title = el.getElement (args[0]).getText ();
				link = el.getElement (args[1]).getText ();
				description = clean (el.getElement (args[2]).getText ());
				mRSSListener.itemParsed (title, link, description);
			
			if (debug) {
				System.err.println ("Element name: " + el.getName ());
				System.err.println ("\t" + title);
				System.err.println ("\t" + link);
				System.err.println ("\t" + description);
				}
			}
		}
	}
}
/*
	The crudest way to strip tags you ever saw! ;-)
*/
private String clean (String string)
{
	String out;
	try {
		out = string.substring (0, string.indexOf ("<"));
	} catch (IndexOutOfBoundsException noTags) {				// No tags found
		out = string;
		}
		
	return out;
}

public void setRSSListener (RSSListener listener) { mRSSListener = listener; }
  
}